/*
 * Licensed to CRATE Technology GmbH ("Crate") under one or more contributor
 * license agreements.  See the NOTICE file distributed with this work for
 * additional information regarding copyright ownership.  Crate licenses
 * this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 *
 * However, if you have executed another commercial license agreement
 * with Crate these terms will supersede the license and you may use the
 * software solely pursuant to the terms of the relevant commercial agreement.
 */

package io.crate.operation;

/**
 * Marker for objects which can be an upstream of a {@link RowDownstream}.
 */
public interface RowUpstream {

    /**
     * tells the RowUpstream to stop sending rows until resume is called.
     *
     * pause() must be called in the same thread that is also calling setNextRow
     */
    void pause();

    /**
     * tells the RowUpstream that it should resume sending rows.
     */
    void resume(boolean async);


    /**
     * tells the RowUpstream that it should push all rows again
     */
    void repeat();
}
